package com.futu.course.es.Service.impl;

import com.alibaba.fastjson.JSON;
import com.futu.course.course.domain.Course;
import com.futu.course.course.service.impl.CourseServiceImpl;
import com.futu.course.es.Service.ESService;
import com.futu.course.es.domain.ESCourse;
import com.futu.course.es.domain.MongoDBCourse;
import org.elasticsearch.action.admin.indices.exists.indices.IndicesExistsRequest;
import org.elasticsearch.action.admin.indices.exists.indices.IndicesExistsResponse;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.support.WriteRequest;
import org.elasticsearch.client.IndicesClient;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.indices.CreateIndexRequest;
import org.elasticsearch.client.indices.CreateIndexResponse;
import org.elasticsearch.client.indices.GetIndexRequest;
import org.elasticsearch.common.bytes.BytesReference;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MatchAllQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.reindex.DeleteByQueryRequest;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.xcontent.XContentBuilder;
import org.elasticsearch.xcontent.XContentFactory;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;

/**
 * @author "杨旭朋"
 * @ClassName: ESCourseService
 * @date 2025年05月06日 15:54
 */
@Service
public class ESCourseService {
    @Autowired
    RestHighLevelClient client;
    @Autowired
    CourseServiceImpl courseService;
    @Autowired
    ESService esService;

    /**
     * @title: search
     * @desc: ES搜索指引
     * @params (String name)
     * @return: ArrayList<String>
     * @author: 杨旭朋
     * @date: 2025/5/6 16:29
     */
    public ArrayList<String> search(String name) throws IOException {
        // 创建搜索请求
        SearchRequest searchRequest = new SearchRequest("course");
        // 创建搜索容器
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        // 创建布尔查询器
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery()
                // 创建条件，在条件中添加 match 查询，使用 ik 分词器
                .must(QueryBuilders.matchQuery("name", name));
        // 设置查询的最大条数
        searchSourceBuilder.size(5);
        // 将搜索容器添加搜索条件
        searchSourceBuilder.query(boolQueryBuilder);
        // 添加数据为搜索容器
        searchRequest.source(searchSourceBuilder);
        // 执行请求，并获取响应
        SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);
        System.out.println(searchRequest);
        // 返回搜索结果
        SearchHit[] hits = search.getHits().getHits();
        ArrayList<String> list = new ArrayList<>();
        for (SearchHit hit : hits) {
            Map<String, Object> sourceAsMap = hit.getSourceAsMap();
            list.add((String) sourceAsMap.get("name"));
        }
        return list;
    }

    /**
     * @title: MysqlInterES
     * @desc: 将mysql中的数据与ES进行同步
     * @params ()
     * @return: Result
     * @author: 杨旭朋
     * @date: 2025/5/6 16:50
     */
    @Transactional
    public void MysqlInterES() throws IOException {
//        创建索引
        this.checkAndCreateIndex();
        // ES删除所有数据
        DeleteByQueryRequest request = new DeleteByQueryRequest("course");
        // 使用 MatchAllQuery 匹配所有文档
        request.setQuery(QueryBuilders.matchAllQuery());
        client.deleteByQuery(request, RequestOptions.DEFAULT);
        // 查询数据库中所有的课程
        List<Course> list = courseService.list();
        List<ESCourse> esCourses = new ArrayList<>();

        // 初始化数据
        for (Course course : list) {
            System.out.println(course);
            ESCourse esCourse = new ESCourse();
            esCourse.setId(Long.valueOf(String.valueOf(course.getId())));
            esCourse.setName(course.getTitle());
            esCourse.setCategoryId(Long.valueOf(course.getCategoryId()));
            esCourse.setCoverImg(course.getCoverImg());
            esCourse.setPrice(course.getPrice());
            esCourses.add(esCourse);
            System.out.println(esCourse);
        }
        // 将数据添加到ES中
        BulkRequest bulkRequest = new BulkRequest("course");
        for (ESCourse esCourse : esCourses) {
            String string = JSON.toJSONString(esCourse);
            IndexRequest indexRequest = new IndexRequest()
                    .id(esCourse.getId().toString())
                    .source(string, XContentType.JSON);
            bulkRequest.add(indexRequest);
        }
        BulkResponse bulkResponse = client.bulk(bulkRequest, RequestOptions.DEFAULT);
    }

    /**
     * @title: CourseList
     * @desc: 分类查询所有
     * @params int categoryId
     * @return: HashSet<ESCourse>
     * @author: 杨旭朋
     * @date: 2025/5/7 17:09
     */
    public HashSet<ESCourse> CourseList(int categoryId) throws IOException {
        SearchRequest searchRequest = new SearchRequest("course");
        // 创建搜索容器
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        HashSet<ESCourse> esCourses = new HashSet<>();
        if (categoryId != 0) {
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery()
                    // 创建条件在条件中添加 term 查询
                    .must(QueryBuilders.termQuery("categoryId", categoryId));
            // 将搜索容器添加搜索条件
            searchSourceBuilder.query(boolQueryBuilder);
            // 添加数据为搜索容器
            searchRequest.source(searchSourceBuilder);
            try {
                // 执行请求，并获取响应
                SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);
                System.out.println(searchRequest);
                SearchHit[] hits = search.getHits().getHits();
                for (SearchHit hit : hits) {
                    ESCourse esCourse = new ESCourse();
                    Map<String, Object> sourceAsMap = hit.getSourceAsMap();

                    // 处理可能的类型转换问题
                    Object idObj = sourceAsMap.get("id");
                    if (idObj instanceof Integer) {
                        esCourse.setId(Long.valueOf(String.valueOf(((Integer) idObj).longValue())));
                    } else if (idObj instanceof Long) {
                        esCourse.setId(Long.valueOf(String.valueOf((Long) idObj)));
                    }

                    esCourse.setName((String) sourceAsMap.get("name"));

                    Object priceObj = sourceAsMap.get("price");
                    if (priceObj instanceof Number) {
                        esCourse.setPrice(BigDecimal.valueOf(((Number) priceObj).doubleValue()));
                    }

                    Object categoryIdObj = sourceAsMap.get("categoryId");
                    if (categoryIdObj instanceof Integer) {
                        esCourse.setCategoryId(((Integer) categoryIdObj).longValue());
                    } else if (categoryIdObj instanceof Long) {
                        esCourse.setCategoryId((Long) categoryIdObj);
                    }

                    esCourse.setCoverImg((String) sourceAsMap.get("coverImg"));
                    esCourses.add(esCourse);
                }
            } catch (IOException e) {
                System.err.println("执行搜索请求时出现异常: ");
                e.printStackTrace();
            }
        } else {
            // 创建 MatchAllQueryBuilder 并设置 boost 值
            MatchAllQueryBuilder matchAllQuery = QueryBuilders.matchAllQuery();
            matchAllQuery.boost(1.0f);
            searchSourceBuilder.query(matchAllQuery);
            // 将 SearchSourceBuilder 设置到 SearchRequest 中
            searchRequest.source(searchSourceBuilder);
            // 执行搜索请求
            SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
            SearchHit[] hits = searchResponse.getHits().getHits();
            for (SearchHit hit : hits) {
                ESCourse esCourse = new ESCourse();
                Map<String, Object> sourceAsMap = hit.getSourceAsMap();

                // 处理可能的类型转换问题
                Object idObj = sourceAsMap.get("id");
                if (idObj instanceof Integer) {
                    esCourse.setId(Long.valueOf(String.valueOf(((Integer) idObj).longValue())));
                } else if (idObj instanceof Long) {
                    esCourse.setId(Long.valueOf(String.valueOf((Long) idObj)));
                }

                esCourse.setName((String) sourceAsMap.get("name"));

                Object priceObj = sourceAsMap.get("price");
                if (priceObj instanceof Number) {
                    esCourse.setPrice(BigDecimal.valueOf(((Number) priceObj).doubleValue()));
                }

                Object categoryIdObj = sourceAsMap.get("categoryId");
                if (categoryIdObj instanceof Integer) {
                    esCourse.setCategoryId(((Integer) categoryIdObj).longValue());
                } else if (categoryIdObj instanceof Long) {
                    esCourse.setCategoryId((Long) categoryIdObj);
                }

                esCourse.setCoverImg((String) sourceAsMap.get("coverImg"));
                esCourses.add(esCourse);
            }
            // 处理搜索结果
        }
        return esCourses;
    }
    /**
     * @title: SelCourse
     * @desc:  搜索并将搜索记录写入到MongoDB中
     * @params ()
     * @return: Result
     * @author: 杨旭朋
     * @date: 2025/5/8 8:35
     */
    public HashSet<ESCourse> SelCourse(String name) throws IOException {
        // 创建搜索请求
        SearchRequest searchRequest = new SearchRequest("course");
        // 创建搜索容器
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        // 创建布尔查询器
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery()
                // 创建条件，在条件中添加 match 查询，使用 ik 分词器
                .must(QueryBuilders.matchQuery("name", name));
        // 将搜索容器添加搜索条件
        searchSourceBuilder.query(boolQueryBuilder);
        // 添加数据为搜索容器
        searchRequest.source(searchSourceBuilder);
        // 执行请求，并获取响应
        SearchResponse search = client.search(searchRequest, RequestOptions.DEFAULT);
        System.out.println(searchRequest);
        // 返回搜索结果
        SearchHit[] hits = search.getHits().getHits();
        HashSet<ESCourse> esCourses = new HashSet<>();
        for (SearchHit hit : hits) {
            ESCourse esCourse = new ESCourse();
            Map<String, Object> sourceAsMap = hit.getSourceAsMap();

            // 处理可能的类型转换问题
            Object idObj = sourceAsMap.get("id");
            if (idObj instanceof Integer) {
                esCourse.setId(Long.valueOf(String.valueOf(((Integer) idObj).longValue())));
            } else if (idObj instanceof Long) {
                esCourse.setId(Long.valueOf(String.valueOf((Long) idObj)));
            }

            esCourse.setName((String) sourceAsMap.get("name"));

            Object priceObj = sourceAsMap.get("price");
            if (priceObj instanceof Number) {
                esCourse.setPrice(BigDecimal.valueOf(((Number) priceObj).doubleValue()));
            }

            Object categoryIdObj = sourceAsMap.get("categoryId");
            if (categoryIdObj instanceof Integer) {
                esCourse.setCategoryId(((Integer) categoryIdObj).longValue());
            } else if (categoryIdObj instanceof Long) {
                esCourse.setCategoryId((Long) categoryIdObj);
            }

            esCourse.setCoverImg((String) sourceAsMap.get("coverImg"));
            esCourses.add(esCourse);
        }
        this.SaveMongoDB(name);
        return esCourses;
    }
    /**
     * @title: SaveMongoDB
     * @desc:  保存聊天记录到MongoDB中
     * @params String name
     * @return: void
     * @author: 杨旭朋
     * @date: 2025/5/8 9:14
     */
    public void SaveMongoDB(String name){
        List<MongoDBCourse> all = esService.findAll();
        int i = 0;
        for (MongoDBCourse esCourse : all) {
            if (esCourse.getName().equals(name)&&esCourse.getUid().equals(1L)){
                MongoDBCourse byId = esService.findById(esCourse.getId().toString()).get();
                byId.setCreateTime(new Date());
                esService.save(byId);
                i++;
            }
        }
        if (i == 0){
            MongoDBCourse mongoDBCourse = new MongoDBCourse();
            String string = UUID.randomUUID().toString();
            mongoDBCourse.setId(string);
            mongoDBCourse.setName(name);
            mongoDBCourse.setCreateTime(new Date());
            mongoDBCourse.setUid(1L);
            esService.save(mongoDBCourse);
        }
    }
    /**
     * @title: checkAndCreateIndex
     * @desc:  数据初始化
     * @params ()
     * @return: Result
     * @author: 杨旭朋
     * @date: 2025/5/8 15:25
     */
    public boolean checkAndCreateIndex() throws IOException {
        IndicesClient indicesClient = client.indices();
        // 检查索引是否存在
        GetIndexRequest getIndexRequest = new GetIndexRequest("course");
        boolean exists = indicesClient.exists(getIndexRequest, RequestOptions.DEFAULT);

        if (!exists) {
            // 索引不存在，创建索引
            CreateIndexRequest createRequest = new CreateIndexRequest("course");
            // 设置索引映射
            XContentBuilder mappingBuilder = XContentFactory.jsonBuilder();
            mappingBuilder.startObject()
                    .startObject("properties")
                    .startObject("id")
                    .field("type", "long")
                    .endObject()
                    .startObject("name")
                    .field("type", "text")
                    .field("analyzer", "ik_max_word")
                    .endObject()
                    .startObject("price")
                    .field("type", "float")
                    .endObject()
                    .startObject("category_id")
                    .field("type", "long")
                    .endObject()
                    .startObject("cover_img")
                    .field("type", "keyword")
                    .endObject()
                    .endObject()
                    .endObject();
            createRequest.mapping(mappingBuilder);

            CreateIndexResponse createResponse = indicesClient.create(createRequest, RequestOptions.DEFAULT);
            if (createResponse.isAcknowledged()) {
                System.out.println("索引course创建成功");
                return true;
            } else {
                System.out.println("索引course创建失败");
                return false;
            }
        }
        System.out.println("索引course已存在");
        return true;
    }

}